#==============================================================================
# package      DE_EPAGES::TrustedShops::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::TrustedShops::UI::Shop;
use base qw (DE_EPAGES::Presentation::UI::Object);

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( AddDuration ElementOf isValid trim );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadClassByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::TrustedShops::API::Constants qw (
    %TS_PAYMENT_MAPPING
    @TS_PAYMENT_METHODS
    TS_ID_LENGTH
);
use DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler;
use DE_EPAGES::TrustedShops::API::RemoteTrustedShops qw ( GetRequestState );
use DE_EPAGES::TrustedShops::API::Tools qw (
    LogTrustedShops
    SendShopChangesToUnity
    TrustedShopsLogContext
    UnusedCertificateLanguages
);
#========================================================================================
# function     View
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->View();
#----------------------------------------------------------------------------------------
# description  UI Action to view a object without any work.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewTrustedShopsCertificates', { 'Servlet' => $Servlet });

    $Servlet->vars('PossibleLanguagesForCertificates', UnusedCertificateLanguages($Shop));
    $self->SUPER::View($Servlet);

    return;
}
#==============================================================================
sub SaveCertificates {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $Shop = $Servlet->object();
    my $TsID = $Form->formValue('NewAlias');
    my $IsExcellence = $Form->formValue('IsExcellence');
    my $CertificationLanguage = $Form->formValue('CertificationLanguage');
    my $Folder = $Shop->folder('TrustedShopsCertificates');

    if (isValid($TsID)){
        $TsID = $self->_checkTSID($TsID);
    } else {
        return;
    }

    $self->_checkTrustedShopsID($TsID, $Form);
    $self->_checkTrustedShopsIDExists($TsID, $Form, $Folder);

    my $TrustedShopsCertificateHandler = $self->_getTrustedShopsCertificateHandler($Shop, $TsID, $IsExcellence, $CertificationLanguage);
    my $TSCertificateObjectID;
    eval {
       $TSCertificateObjectID = $TrustedShopsCertificateHandler->createCertificateObject();
    };

    if (ExistsError()){
        my $Error = GetError();
        $self->_createFormError($Form, $Error->shortMessage());
    }
    else {
        $Servlet->vars('HasCertificateJustBeenAdded', 1);
    }

    my $TrustedShopsCertificate = $TrustedShopsCertificateHandler->getCertificateObject($TSCertificateObjectID);

    if ($Shop->get('IsUnityRegistered')) {
        $self->_SendShopChangesToUnity($Shop, $TrustedShopsCertificate);
    }

    $self->_LogTrustedShops('TrustedShopsID', $TSCertificateObjectID, 'SaveCertificates');

    return;
}

#==============================================================================
sub _createFormError {
    my $self = shift;
    my ($Form, $Reason) = @_;
                $Form->executeFormError({
                    'Name'   => 'NewAlias',
                    'Form'   => 'SaveTrustedShops',
                    'Reason' => $Reason
                });
    return;
}

#==============================================================================
sub _checkTSID {
    my $self = shift;
    my ($TsID) = @_;

    return (trim($TsID))[0];

}
#==============================================================================
sub _checkTrustedShopsID {
    my $self = shift;
    my ($TsID, $Form) = @_;

    unless (length($TsID) == TS_ID_LENGTH  &&  $TsID =~ /^X/) {
        $Form->executeFormError({
            'Name'   => 'NewAlias',
            'Form'   => 'SaveTrustedShops',
            'Reason' => 'INVALID_FORMAT'
        });
    }

    return;

}
#==============================================================================
sub _checkTrustedShopsIDExists {
    my $self = shift;
    my ($TsID, $Form, $Folder) = @_;

    if ($Folder->existsChild($TsID)) {
        $Form->executeFormError({
            'Name'   => 'NewAlias',
            'Form'   => 'SaveTrustedShops',
            'Reason' => 'OBJECT_EXISTS'
        });
    }

    return;
}
#==============================================================================
sub ViewPaymentMethods {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    # set ts payment method for known payment types if not set yet - can be changed by merchant later on
    foreach my $PaymentMethod (@{$Shop->get('PaymentMethods')}) {
        next if defined($PaymentMethod->get('TrustedShopsPaymentMethod'));
        my $PaymentType = $PaymentMethod->get('PaymentType');
        if (exists $TS_PAYMENT_MAPPING{$PaymentType->alias}) {
            $PaymentMethod->set({'TrustedShopsPaymentMethod' => $TS_PAYMENT_MAPPING{$PaymentType->alias}});
        }
    }

    $Servlet->vars('TrustedShopsPaymentMethods', [map { {'Name' => $_} } @TS_PAYMENT_METHODS]);

    return;
}

#==============================================================================
sub SavePaymentMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    my $hValues = $Form->form($Shop, 'TrustedShopsPaymentMethods');

    #--- additional checks of form values
    my $ItemNo = -1;
    my @ValuesToSet; # ( [ $PaymentMethod, $TrustedShopsPaymentMethodID ], ... ] )
    foreach my $hEntry (@{$hValues->{'PaymentMethods'}}) {
        $ItemNo++;
        if (isValid($hEntry->{'TrustedShopsPaymentMethod'}) && !ElementOf($hEntry->{'TrustedShopsPaymentMethod'}, @TS_PAYMENT_METHODS)) {
            $Form->addFormError({
                'Reason'   => 'InvalidTrustedShopsPaymentMethod',
                'Name'     => 'TrustedShopsPaymentMethod',
                'LoopName' => 'PaymentMethods',
                'Index'    => $ItemNo,
                'Form'     => 'TrustedShopsPaymentMethods'
            });
            next;
        }
        unless (ExistsObject($hEntry->{'PaymentMethodID'})) {
            $Form->addFormError({
                'Reason'   => 'ObjectNotFound',
                'Name'     => 'PaymentMethodID',
                'LoopName' => 'PaymentMethods',
                'Index'    => $ItemNo,
                'Form'     => 'TrustedShopsPaymentMethods'
            });
            next;
        }

        my $PaymentMethod = $Shop->loadObject($hEntry->{'PaymentMethodID'}, 'PaymentMethod');
        push @ValuesToSet, [ $PaymentMethod, $hEntry->{'TrustedShopsPaymentMethod'} ];
    }
    $Form->executeFormError;

    # save
    foreach (@ValuesToSet) {
        my ($PaymentMethod, $TrustedShopsPaymentMethod) = @$_;
        $PaymentMethod->set({'TrustedShopsPaymentMethod' => $TrustedShopsPaymentMethod});
    }

    return;
}

#========================================================================================
# function     ViewTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Simple user interface for searching transactions if
#               <function SearchTransactions> failed.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hPagerVars | pager variables that can be used inside a template (cf. $Pager-><function page>) | ref.hash
#========================================================================================
sub ViewTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my %SearchForm = ();

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'TransTime');
        $Servlet->params('OrderDesc', 1);
        $SearchForm{'OrderBy'}   = 'TransTime';
        $SearchForm{'OrderDesc'} = 1;
    }

    #--- add string fields if submitted
    foreach my $field (qw(TransID ApplicationID)) {
        $SearchForm{$field} = $Servlet->params($field) if (defined $Servlet->params($field));
    }

    #--- add time fields if submitted
    foreach my $field (qw(TransTimeFrom TransTimeTo)) {
        eval { ## no critic qw( RequireExistsError ) # ignore invalid date strings
            my $Value = $Servlet->params($field);
            $SearchForm{$field} = $Servlet->form()->convertDateTime($Value, {'Type'=>'reg_date'}) if defined $Value;
        };
    }

    #--- remove empty entries
    delete $SearchForm{$_} foreach grep { $SearchForm{$_} !~ /\S/ } keys %SearchForm;

    #--- do the search
    return $self->_ExecuteTransactionSearch($Servlet, \%SearchForm);
}

#========================================================================================
# function     SearchTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SearchTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Searches for transactions of the current payment type.
#               The TLE variable #Pager contains the search results.
#               See also <function TransactionPaymentName>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SearchTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Shop, 'SearchTrustedShopsTransactions');
    $self->_ExecuteTransactionSearch($Servlet, $SearchForm);
}

#========================================================================================
# function     _ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hPagerVars | pager variables that can be used inside a template (cf. $Pager-><function page>) | ref.hash
#========================================================================================
sub _ExecuteTransactionSearch {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'Search'} = 'Transactions' unless exists $SearchForm->{'Search'};
    my $Shop = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('LineItemTSProtectionItem'), $SearchForm->{'Search'});

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        }
        elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        }
        elsif ($Parameter eq 'TransTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);

        }
        else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-12969: use original input values in pager URL
        if ($Parameter =~ /^TransTime(From|To)$/) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

#========================================================================================
sub UpdateTransactions {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    my @ToUpdate = $Form->value('ListObjectID');
    return if (0 == scalar @ToUpdate);

    my $aWarnings = BlockWarnings( sub {
        foreach my $objectID(@ToUpdate) {
            my $LineItemTS = $Shop->loadObject($objectID, 'LineItemTSProtectionItem');
            my $ret = GetRequestState($LineItemTS);
            $LineItemTS->set({'GuaranteeNo' => $ret});
            if (defined $ret && $ret < 0) {
                Warning('TrustedShopsUpdateFailed', {
                    'ApplicationID' => $LineItemTS->get('ApplicationID'),
                    'ErrorCode'     => $ret
                });
            }
        }
    }, ['TrustedShopsUpdateFailed']);

    $Form->mapFormError({
        'Form'  => 'UpdateTrustedShopsTransactions',
        'Name'  => 'UpdateTrustedShopsTransactions'
    }, $_) foreach @$aWarnings;
    $Form->executeFormError;

    return;
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _getTrustedShopsCertificateHandler {
    my $self = shift;
    my ($Shop, $TrustedShopsID, $IsExcellence, $CertificationLanguage) = @_;

    my $TrustedShopsCertificateHandler = DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler->new(
        $Shop,
        $TrustedShopsID,
        $IsExcellence,
        $CertificationLanguage
    );

    return $TrustedShopsCertificateHandler;
}

sub _LogTrustedShops {
    my $self = shift;
    my ($Name, $DebugStuff, $Section) = @_;

    LogTrustedShops($Name, $DebugStuff, $Section);

    return;
}


sub _SendShopChangesToUnity {
    my $self = shift;
    my ($Shop, $TrustedShopsCertificate) = @_;

    TrustedShopsLogContext($TrustedShopsCertificate, sub {
        my $TrustbadgeCode = $TrustedShopsCertificate->get('TrustbadgeCardCoding');
        my $hParams = {
            'Object' => $Shop,
            'Values' => {'TrustbadgeCardCoding' => $TrustbadgeCode},
            'Language' => $TrustedShopsCertificate->get('CertificationLanguage')
        };
        SendShopChangesToUnity($hParams);
    });

    return;
}

1;
